@@ -22,17 +22,19 @@ import gtk
 
 from sugar import _sugarext
 
+
 _PLACE_TRIALS = 20
 _MAX_WEIGHT = 255
 _REFRESH_RATE = 200
 _MAX_COLLISIONS_PER_REFRESH = 20
 
+
 class Grid(_sugarext.Grid):
     __gsignals__ = {
-        'child-changed' : (gobject.SIGNAL_RUN_FIRST,
-                           gobject.TYPE_NONE,
-                           ([gobject.TYPE_PYOBJECT]))
+        'child-changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
+                          ([gobject.TYPE_PYOBJECT])),
     }
+
     def __init__(self, width, height):
         gobject.GObject.__init__(self)
 
@@ -185,7 +187,8 @@ class Grid(_sugarext.Grid):
         for c in self._children:
             intersection = child_rect.intersect(self._child_rects[c])
             if c != child and intersection.width > 0:
-                if c not in self._locked_children and c not in self._collisions:
+                if (c not in self._locked_children and
+                    c not in self._collisions):
                     collision_found = True
                     self._collisions.append(c)
 
