@@ -22,10 +22,15 @@ import gtk
 import dbus
 
 from jarabe.model import network
-from jarabe.model.network import Secrets
+
 
 IW_AUTH_ALG_OPEN_SYSTEM = 'open'
-IW_AUTH_ALG_SHARED_KEY  = 'shared'
+IW_AUTH_ALG_SHARED_KEY = 'shared'
+
+WEP_PASSPHRASE = 1
+WEP_HEX = 2
+WEP_ASCII = 3
+
 
 def string_is_hex(key):
     is_hex = True
@@ -34,6 +39,7 @@ def string_is_hex(key):
             is_hex = False
     return is_hex
 
+
 def string_is_ascii(string):
     try:
         string.encode('ascii')
@@ -41,12 +47,14 @@ def string_is_ascii(string):
     except UnicodeEncodeError:
         return False
 
+
 def string_to_hex(passphrase):
     key = ''
     for c in passphrase:
         key += '%02x' % ord(c)
     return key
 
+
 def hash_passphrase(passphrase):
     # passphrase must have a length of 64
     if len(passphrase) > 64:
@@ -57,18 +65,18 @@ def hash_passphrase(passphrase):
     passphrase = hashlib.md5(passphrase).digest()
     return string_to_hex(passphrase)[:26]
 
+
 class CanceledKeyRequestError(dbus.DBusException):
     def __init__(self):
         dbus.DBusException.__init__(self)
         self._dbus_error_name = network.NM_SETTINGS_IFACE + '.CanceledError'
 
-class KeyDialog(gtk.Dialog):
-    def __init__(self, ssid, flags, wpa_flags, rsn_flags, dev_caps, settings,
-                 response):
-        gtk.Dialog.__init__(self, flags=gtk.DIALOG_MODAL)
-        self.set_title("Wireless Key Required")
 
-        self._settings = settings
+class KeyDialog(gtk.Dialog):
+    def __init__(self, ssid, flags, wpa_flags, rsn_flags, dev_caps, response):
+        gtk.Dialog.__init__(self, flags=gtk.DIALOG_MODAL)
+        self.set_title('Wireless Key Required')
+
         self._response = response
         self._entry = None
         self._ssid = ssid
@@ -79,8 +87,9 @@ class KeyDialog(gtk.Dialog):
 
         self.set_has_separator(False)
 
-        label = gtk.Label("A wireless encryption key is required for\n" \
-                          " the wireless network '%s'." % self._ssid)
+        display_name = network.ssid_to_display_name(ssid)
+        label = gtk.Label(_("A wireless encryption key is required for\n"
+                            " the wireless network '%s'.") % (display_name, ))
         self.vbox.pack_start(label)
 
         self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
@@ -108,21 +117,17 @@ class KeyDialog(gtk.Dialog):
     def get_response_object(self):
         return self._response
 
-WEP_PASSPHRASE = 1
-WEP_HEX = 2
-WEP_ASCII = 3
 
 class WEPKeyDialog(KeyDialog):
-    def __init__(self, ssid, flags, wpa_flags, rsn_flags, dev_caps, settings,
-                 response):
+    def __init__(self, ssid, flags, wpa_flags, rsn_flags, dev_caps, response):
         KeyDialog.__init__(self, ssid, flags, wpa_flags, rsn_flags,
-                           dev_caps, settings, response)
+                           dev_caps, response)
 
         # WEP key type
         self.key_store = gtk.ListStore(str, int)
-        self.key_store.append(["Passphrase (128-bit)", WEP_PASSPHRASE])
-        self.key_store.append(["Hex (40/128-bit)", WEP_HEX])
-        self.key_store.append(["ASCII (40/128-bit)", WEP_ASCII])
+        self.key_store.append(['Passphrase (128-bit)', WEP_PASSPHRASE])
+        self.key_store.append(['Hex (40/128-bit)', WEP_HEX])
+        self.key_store.append(['ASCII (40/128-bit)', WEP_ASCII])
 
         self.key_combo = gtk.ComboBox(self.key_store)
         cell = gtk.CellRendererText()
@@ -132,7 +137,7 @@ class WEPKeyDialog(KeyDialog):
         self.key_combo.connect('changed', self._key_combo_changed_cb)
 
         hbox = gtk.HBox()
-        hbox.pack_start(gtk.Label(_("Key Type:")))
+        hbox.pack_start(gtk.Label(_('Key Type:')))
         hbox.pack_start(self.key_combo)
         hbox.show_all()
         self.vbox.pack_start(hbox)
@@ -142,8 +147,8 @@ class WEPKeyDialog(KeyDialog):
 
         # WEP authentication mode
         self.auth_store = gtk.ListStore(str, str)
-        self.auth_store.append(["Open System", IW_AUTH_ALG_OPEN_SYSTEM])
-        self.auth_store.append(["Shared Key", IW_AUTH_ALG_SHARED_KEY])
+        self.auth_store.append(['Open System', IW_AUTH_ALG_OPEN_SYSTEM])
+        self.auth_store.append(['Shared Key', IW_AUTH_ALG_SHARED_KEY])
 
         self.auth_combo = gtk.ComboBox(self.auth_store)
         cell = gtk.CellRendererText()
@@ -152,7 +157,7 @@ class WEPKeyDialog(KeyDialog):
         self.auth_combo.set_active(0)
 
         hbox = gtk.HBox()
-        hbox.pack_start(gtk.Label(_("Authentication Type:")))
+        hbox.pack_start(gtk.Label(_('Authentication Type:')))
         hbox.pack_start(self.auth_combo)
         hbox.show_all()
 
@@ -179,15 +184,13 @@ class WEPKeyDialog(KeyDialog):
 
     def print_security(self):
         (key, auth_alg) = self._get_security()
-        print "Key: %s" % key
-        print "Auth: %d" % auth_alg
+        print 'Key: %s' % key
+        print 'Auth: %d' % auth_alg
 
     def create_security(self):
         (key, auth_alg) = self._get_security()
-        secrets = Secrets(self._settings)
-        secrets.wep_key = key
-        secrets.auth_alg = auth_alg
-        return secrets
+        wsec = {'wep-key0': key, 'auth-alg': auth_alg}
+        return {'802-11-wireless-security': wsec}
 
     def _update_response_sensitivity(self, ignored=None):
         key = self._entry.get_text()
@@ -209,15 +212,15 @@ class WEPKeyDialog(KeyDialog):
 
         self.set_response_sensitive(gtk.RESPONSE_OK, valid)
 
+
 class WPAKeyDialog(KeyDialog):
-    def __init__(self, ssid, flags, wpa_flags, rsn_flags, dev_caps, settings,
-                 response):
+    def __init__(self, ssid, flags, wpa_flags, rsn_flags, dev_caps, response):
         KeyDialog.__init__(self, ssid, flags, wpa_flags, rsn_flags,
-                           dev_caps, settings, response)
+                           dev_caps, response)
         self.add_key_entry()
 
         self.store = gtk.ListStore(str)
-        self.store.append([_("WPA & WPA2 Personal")])
+        self.store.append([_('WPA & WPA2 Personal')])
 
         self.combo = gtk.ComboBox(self.store)
         cell = gtk.CellRendererText()
@@ -226,7 +229,7 @@ class WPAKeyDialog(KeyDialog):
         self.combo.set_active(0)
 
         self.hbox = gtk.HBox()
-        self.hbox.pack_start(gtk.Label(_("Wireless Security:")))
+        self.hbox.pack_start(gtk.Label(_('Wireless Security:')))
         self.hbox.pack_start(self.combo)
         self.hbox.show_all()
 
@@ -246,26 +249,25 @@ class WPAKeyDialog(KeyDialog):
             from subprocess import Popen, PIPE
             p = Popen(['wpa_passphrase', ssid, key], stdout=PIPE)
             for line in p.stdout:
-                if line.strip().startswith("psk="):
+                if line.strip().startswith('psk='):
                     real_key = line.strip()[4:]
             if p.wait() != 0:
-                raise RuntimeError("Error hashing passphrase")
+                raise RuntimeError('Error hashing passphrase')
             if real_key and len(real_key) != 64:
                 real_key = None
 
         if not real_key:
-            raise RuntimeError("Invalid key")
+            raise RuntimeError('Invalid key')
 
         return real_key
 
     def print_security(self):
         key = self._get_security()
-        print "Key: %s" % key
+        print 'Key: %s' % key
 
     def create_security(self):
-        secrets = Secrets(self._settings)
-        secrets.psk = self._get_security()
-        return secrets
+        wsec = {'psk': self._get_security()}
+        return {'802-11-wireless-security': wsec}
 
     def _update_response_sensitivity(self, ignored=None):
         key = self._entry.get_text()
@@ -281,21 +283,19 @@ class WPAKeyDialog(KeyDialog):
         self.set_response_sensitive(gtk.RESPONSE_OK, valid)
         return False
 
-def create(ssid, flags, wpa_flags, rsn_flags, dev_caps, settings, response):
+
+def create(ssid, flags, wpa_flags, rsn_flags, dev_caps, response):
     if wpa_flags == network.NM_802_11_AP_SEC_NONE and \
             rsn_flags == network.NM_802_11_AP_SEC_NONE:
         key_dialog = WEPKeyDialog(ssid, flags, wpa_flags, rsn_flags,
-                                  dev_caps, settings, response)
+                                  dev_caps, response)
     else:
         key_dialog = WPAKeyDialog(ssid, flags, wpa_flags, rsn_flags,
-                                  dev_caps, settings, response)
+                                  dev_caps, response)
 
-    key_dialog.connect("response", _key_dialog_response_cb)
-    key_dialog.connect("destroy", _key_dialog_destroy_cb)
+    key_dialog.connect('response', _key_dialog_response_cb)
     key_dialog.show_all()
 
-def _key_dialog_destroy_cb(key_dialog, data=None):
-    _key_dialog_response_cb(key_dialog, gtk.RESPONSE_CANCEL)
 
 def _key_dialog_response_cb(key_dialog, response_id):
     response = key_dialog.get_response_object()
@@ -303,15 +303,15 @@ def _key_dialog_response_cb(key_dialog, response_id):
     if response_id == gtk.RESPONSE_OK:
         secrets = key_dialog.create_security()
 
-    if response_id in [gtk.RESPONSE_CANCEL, gtk.RESPONSE_NONE]:
+    if response_id in [gtk.RESPONSE_CANCEL, gtk.RESPONSE_NONE,
+                       gtk.RESPONSE_DELETE_EVENT]:
         # key dialog dialog was canceled; send the error back to NM
         response.set_error(CanceledKeyRequestError())
     elif response_id == gtk.RESPONSE_OK:
         if not secrets:
-            raise RuntimeError("Invalid security arguments.")
+            raise RuntimeError('Invalid security arguments.')
         response.set_secrets(secrets)
     else:
-        raise RuntimeError("Unhandled key dialog response %d" % response_id)
+        raise RuntimeError('Unhandled key dialog response %d' % response_id)
 
     key_dialog.destroy()
-
