@@ -23,6 +23,7 @@ from sugar.graphics import animator
 from jarabe.model.buddy import get_owner_instance
 from jarabe.view.buddyicon import BuddyIcon
 
+
 class _Animation(animator.Animation):
     def __init__(self, icon, start_size, end_size):
         animator.Animation.__init__(self, 0.0, 1.0)
@@ -33,10 +34,12 @@ class _Animation(animator.Animation):
 
     def next_frame(self, current):
         d = (self.end_size - self.start_size) * current
-        self._icon.props.size = self.start_size + d
+        self._icon.props.size = int(self.start_size + d)
+
 
 class _Layout(gobject.GObject, hippo.CanvasLayout):
     __gtype_name__ = 'SugarTransitionBoxLayout'
+
     def __init__(self):
         gobject.GObject.__init__(self)
         self._box = None
@@ -60,12 +63,12 @@ class _Layout(gobject.GObject, hippo.CanvasLayout):
                            y + (height - child_height) / 2,
                            child_width, child_height, origin_changed)
 
+
 class TransitionBox(hippo.Canvas):
     __gtype_name__ = 'SugarTransitionBox'
 
     __gsignals__ = {
-        'completed': (gobject.SIGNAL_RUN_FIRST,
-                      gobject.TYPE_NONE, ([]))
+        'completed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ([])),
     }
 
     def __init__(self):
