@@ -28,8 +28,12 @@ from sugar.profile import get_profile
 
 from jarabe.util.telepathy import connection_watcher
 
+
 CONNECTION_INTERFACE_BUDDY_INFO = 'org.laptop.Telepathy.BuddyInfo'
 
+_owner_instance = None
+
+
 class BaseBuddyModel(gobject.GObject):
     __gtype_name__ = 'SugarBaseBuddyModel'
 
@@ -89,6 +93,7 @@ class BaseBuddyModel(gobject.GObject):
 
 class OwnerBuddyModel(BaseBuddyModel):
     __gtype_name__ = 'SugarOwnerBuddyModel'
+
     def __init__(self):
         BaseBuddyModel.__init__(self)
 
@@ -101,8 +106,6 @@ class OwnerBuddyModel(BaseBuddyModel):
 
         self.connect('notify::nick', self.__property_changed_cb)
         self.connect('notify::color', self.__property_changed_cb)
-        self.connect('notify::current-activity',
-                     self.__current_activity_changed_cb)
 
         bus = dbus.SessionBus()
         bus.add_signal_receiver(
@@ -129,25 +132,6 @@ class OwnerBuddyModel(BaseBuddyModel):
     def __property_changed_cb(self, buddy, pspec):
         self._sync_properties()
 
-    def __current_activity_changed_cb(self, buddy, pspec):
-        conn_watcher = connection_watcher.get_instance()
-        for connection in conn_watcher.get_connections():
-            if self.props.current_activity is not None:
-                activity_id = self.props.current_activity.activity_id
-                room_handle = self.props.current_activity.room_handle
-            else:
-                activity_id = ''
-                room_handle = 0
-
-            connection[CONNECTION_INTERFACE_BUDDY_INFO].SetCurrentActivity(
-                activity_id,
-                room_handle,
-                reply_handler=self.__set_current_activity_cb,
-                error_handler=self.__error_handler_cb)
-
-    def __set_current_activity_cb(self):
-        logging.debug('__set_current_activity_cb')
-
     def _sync_properties(self):
         conn_watcher = connection_watcher.get_instance()
         for connection in conn_watcher.get_connections():
@@ -180,7 +164,6 @@ class OwnerBuddyModel(BaseBuddyModel):
         return True
 
 
-_owner_instance = None
 def get_owner_instance():
     global _owner_instance
     if _owner_instance is None:
@@ -190,6 +173,7 @@ def get_owner_instance():
 
 class BuddyModel(BaseBuddyModel):
     __gtype_name__ = 'SugarBuddyModel'
+
     def __init__(self, **kwargs):
 
         self._account = None
@@ -225,4 +209,5 @@ class BuddyModel(BaseBuddyModel):
     def set_handle(self, handle):
         self._handle = handle
 
-    handle = gobject.property(type=object, getter=get_handle, setter=set_handle)
+    handle = gobject.property(type=object, getter=get_handle,
+                              setter=set_handle)
