@@ -20,6 +20,7 @@ from gettext import gettext as _
 
 import gtk
 import gconf
+import glib
 import dbus
 
 from sugar.graphics.palette import Palette
@@ -30,6 +31,8 @@ from jarabe.model import shell
 from jarabe.model import friends
 from jarabe.model.session import get_session_manager
 from jarabe.controlpanel.gui import ControlPanel
+import jarabe.desktop.homewindow
+
 
 class BuddyMenu(Palette):
     def __init__(self, buddy):
@@ -38,7 +41,9 @@ class BuddyMenu(Palette):
         buddy_icon = Icon(icon_name='computer-xo',
                           xo_color=buddy.get_color(),
                           icon_size=gtk.ICON_SIZE_LARGE_TOOLBAR)
-        Palette.__init__(self, None, primary_text=buddy.get_nick(),
+        nick = buddy.get_nick()
+        Palette.__init__(self, None,
+                         primary_text=glib.markup_escape_text(nick),
                          icon=buddy_icon)
         self._invite_menu = None
         self._active_activity_changed_hid = None
@@ -86,6 +91,12 @@ class BuddyMenu(Palette):
 
         client = gconf.client_get_default()
 
+        if client.get_bool('/desktop/sugar/show_restart'):
+            item = MenuItem(_('Restart'), 'system-restart')
+            item.connect('activate', self.__reboot_activate_cb)
+            self.menu.append(item)
+            item.show()
+
         if client.get_bool('/desktop/sugar/show_logout'):
             item = MenuItem(_('Logout'), 'system-logout')
             item.connect('activate', self.__logout_activate_cb)
@@ -97,17 +108,18 @@ class BuddyMenu(Palette):
         self.menu.append(item)
         item.show()
 
+    def _quit(self, action):
+        home_window = jarabe.desktop.homewindow.get_instance()
+        home_window.busy_during_delayed_action(action)
+
     def __logout_activate_cb(self, menu_item):
-        session_manager = get_session_manager()
-        session_manager.logout()
+        self._quit(get_session_manager().logout)
 
     def __reboot_activate_cb(self, menu_item):
-        session_manager = get_session_manager()
-        session_manager.reboot()
+        self._quit(get_session_manager().reboot)
 
     def __shutdown_activate_cb(self, menu_item):
-        session_manager = get_session_manager()
-        session_manager.shutdown()
+        self._quit(get_session_manager().shutdown)
 
     def __controlpanel_activate_cb(self, menu_item):
         panel = ControlPanel()
@@ -140,7 +152,7 @@ class BuddyMenu(Palette):
         self._update_invite_menu(activity_model)
 
     def __buddy_notify_nick_cb(self, buddy, pspec):
-        self.set_primary_text(buddy.props.nick)
+        self.set_primary_text(glib.markup_escape_text(buddy.props.nick))
 
     def _make_friend_cb(self, menuitem):
         friends.get_model().make_friend(self._buddy)
